package com.hichina.admin.hichinaadminbackend.controller;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hichina.admin.hichinaadminbackend.config.Constants;
import com.hichina.admin.hichinaadminbackend.mapper.*;
import com.hichina.admin.hichinaadminbackend.model.DTO.HiChinaResult;
import com.hichina.admin.hichinaadminbackend.model.DTO.ProductSkuBatchCreateRequest;
import com.hichina.admin.hichinaadminbackend.model.DTO.ProductSkuBatchCreateResult;
import com.hichina.admin.hichinaadminbackend.model.ProductSkuGroup;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mybatis.spring.boot.test.autoconfigure.AutoConfigureMybatis;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.data.mongo.AutoConfigureDataMongo;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@ActiveProfiles("test")
@AutoConfigureMockMvc
@AutoConfigureMybatis
@AutoConfigureDataMongo
@WithMockUser(username = "admin", roles = {"ADMIN"})
class ProductSkuTestCase {

    @Autowired
    MockMvc mvc;

    @Autowired
    ProductSkuGroupMapper groupMapper;

    @Autowired
    HichinaProductMapper productMapper;

    @Autowired
    ProductSkuIntAttributeMappingMapper productSkuIntAttributeMappingMapper;

    @Autowired
    ProductSkuVarcharAttributeMappingMapper productSkuVarcharAttributeMappingMapper;

    @Autowired
    ProductSkuTimestampAttributeMappingMapper productSkuTimestampAttributeMappingMapper;

    @Test
    @Transactional
    void should_batch_add_product_skus_if_sku_group_not_exist() throws Exception {
        //given:
        var request = new ProductSkuBatchCreateRequest();
        request.setProductName("test");
        request.setProductDescription("test");
        request.setProductTypeId(Constants.TOURPRODUCTTYPE);
        request.setComboCustomPropertyBags(
                List.of(
                        Map.of(
                                "[integer]8cc865ff-b30f-4f00-b426-9e64418e5100", "1",
                                "[string]04a54b65-a1f3-45d9-955c-40cf88f21b82", "vendor",
                                "[string]11cd8b32-c4f6-47db-8b8a-486c992bf43b", "test",
                                "[integer]e228b843-e054-41f8-91dd-19663460df54", "1000",
                                "[integer]c4c845a7-4bef-46d8-a5ad-d72a5464e8b1", "500",
                                "[integer]448406cb-b68f-439e-9da8-148d78ae8404", "0",
                                "[integer]d4825026-3ad8-4861-8e8f-e980b55fe67b", "200",
                                "[string]f3cac6ed-8e87-4fb9-a912-bd87f483b4d5", "testdesc1",
                                "[datestring]f0b807e5-d1a6-4454-a400-7905a4fea492", "2024/3/20;2024/3/21;"
                        ),
                        Map.of(
                                "[integer]8cc865ff-b30f-4f00-b426-9e64418e5100", "1",
                                "[string]04a54b65-a1f3-45d9-955c-40cf88f21b82", "vendor",
                                "[string]11cd8b32-c4f6-47db-8b8a-486c992bf43b", "test2",
                                "[integer]e228b843-e054-41f8-91dd-19663460df54", "1200",
                                "[integer]c4c845a7-4bef-46d8-a5ad-d72a5464e8b1", "500",
                                "[integer]448406cb-b68f-439e-9da8-148d78ae8404", "0",
                                "[integer]d4825026-3ad8-4861-8e8f-e980b55fe67b", "200",
                                "[string]f3cac6ed-8e87-4fb9-a912-bd87f483b4d5", "testdesc1",
                                "[datestring]f0b807e5-d1a6-4454-a400-7905a4fea492", "2024/3/20;2024/3/21;"
                        )
                )
        );

        // when:
        var result = mvc.perform(
                post("/api/v1/productsku/batch")
                        .contentType("application/json")
                        .content((new ObjectMapper()).writeValueAsString(request)));

        // then:
        var content = result.andExpect(status().isOk())
                .andExpect(jsonPath("$.ok").value(true))
                .andReturn().getResponse().getContentAsString();
        HiChinaResult<ProductSkuBatchCreateResult> r = (new ObjectMapper()).readValue(content, new TypeReference<>() {
        });
        ProductSkuBatchCreateResult data = r.getData();
        Assertions.assertEquals(2, data.getSkuIds().size());

        // check group exist
        var groups = groupMapper.findByNameAndProductTypeId("test", Constants.TOURPRODUCTTYPE);
        Assertions.assertEquals(1, groups.size());
        Assertions.assertEquals(1000, groups.get(0).getMinPrice());

        // check sku exist
        for (var skuId : data.getSkuIds()) {
            var sku = productMapper.findBySkuId(skuId);
            Assertions.assertNotNull(sku);
        }

    }

    @Test
    @Transactional
    void should_batch_add_product_skus_if_sku_group_exist() throws Exception {
        //given:
        var g = new ProductSkuGroup("xxxx", "test", Constants.TOURPRODUCTTYPE, new Date(), true, "test", 2000);
        groupMapper.insert(g);

        var request = new ProductSkuBatchCreateRequest();
        request.setProductName("test");
        request.setProductDescription("test");
        request.setProductTypeId(Constants.TOURPRODUCTTYPE);
        request.setComboCustomPropertyBags(
                List.of(
                        Map.of(
                                "[integer]8cc865ff-b30f-4f00-b426-9e64418e5100", "1",
                                "[string]04a54b65-a1f3-45d9-955c-40cf88f21b82", "vendor",
                                "[string]11cd8b32-c4f6-47db-8b8a-486c992bf43b", "test",
                                "[integer]e228b843-e054-41f8-91dd-19663460df54", "1000",
                                "[integer]c4c845a7-4bef-46d8-a5ad-d72a5464e8b1", "500",
                                "[integer]448406cb-b68f-439e-9da8-148d78ae8404", "0",
                                "[integer]d4825026-3ad8-4861-8e8f-e980b55fe67b", "200",
                                "[string]f3cac6ed-8e87-4fb9-a912-bd87f483b4d5", "testdesc1",
                                "[datestring]f0b807e5-d1a6-4454-a400-7905a4fea492", "2024/3/20;2024/3/21;"
                        ),
                        Map.of(
                                "[integer]8cc865ff-b30f-4f00-b426-9e64418e5100", "1",
                                "[string]04a54b65-a1f3-45d9-955c-40cf88f21b82", "vendor",
                                "[string]11cd8b32-c4f6-47db-8b8a-486c992bf43b", "test2",
                                "[integer]e228b843-e054-41f8-91dd-19663460df54", "1200",
                                "[integer]c4c845a7-4bef-46d8-a5ad-d72a5464e8b1", "500",
                                "[integer]448406cb-b68f-439e-9da8-148d78ae8404", "0",
                                "[integer]d4825026-3ad8-4861-8e8f-e980b55fe67b", "200",
                                "[string]f3cac6ed-8e87-4fb9-a912-bd87f483b4d5", "testdesc1",
                                "[datestring]f0b807e5-d1a6-4454-a400-7905a4fea492", "2024/3/20;2024/3/21;"
                        )
                )
        );

        // when:
        var result = mvc.perform(
                post("/api/v1/productsku/batch")
                        .contentType("application/json")
                        .content((new ObjectMapper()).writeValueAsString(request)));

        // then:
        var content = result.andExpect(status().isOk())
                .andExpect(jsonPath("$.ok").value(true))
                .andReturn().getResponse().getContentAsString();
        HiChinaResult<ProductSkuBatchCreateResult> r = (new ObjectMapper()).readValue(content, new TypeReference<>() {
        });
        ProductSkuBatchCreateResult data = r.getData();
        Assertions.assertEquals(2, data.getSkuIds().size());

        // check group exist
        var groups = groupMapper.findByNameAndProductTypeId("test", Constants.TOURPRODUCTTYPE);
        Assertions.assertEquals(1, groups.size());
        Assertions.assertEquals(1000, groups.get(0).getMinPrice());

        // check sku exist
        for (var skuId : data.getSkuIds()) {
            var sku = productMapper.findBySkuId(skuId);
            Assertions.assertNotNull(sku);
        }

    }

    @Test
    @Transactional
    void should_not_insert_any_product_skus_if_error() throws Exception {

    }

    @Test
    void should_not_batch_add_product_skus_if_any_sku_is_invalid() throws Exception {
        //given:
        var request = new ProductSkuBatchCreateRequest();
        request.setProductName("");
        request.setProductDescription("test");
        request.setProductTypeId(Constants.TOURPRODUCTTYPE);

        //when:
        var result = mvc.perform(
                post("/api/v1/productsku/batch")
                        .contentType("application/json")
                        .content((new ObjectMapper()).writeValueAsString(request)));

        //then:
        result.andExpect(status().isBadRequest());

    }
}